﻿using System;
using System.IO.Ports;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

namespace SerialTester
{
    public partial class Form1 : Form
    {
        SerialPort ComPort = null;

        public Form1()
        {
            InitializeComponent();
            showPacket();
            string[] theSerialPorts = SerialPort.GetPortNames();
            cmbPorts.Items.AddRange(theSerialPorts);
            cmbPorts.Text = cmbPorts.Items[0].ToString();
        }

        private void cmdSendPacket_Click(object sender, EventArgs e)
        {
            byte[] packet = createPacket();
            ComPort.Write(packet, 0, packet.Length);
        }

        private byte[] createPacket()
        {
            List<byte> packet = new List<byte>();
            byte packetType = 0;
            byte length = 0;
            if (rbOn.Checked) packetType = 97;
            if (rbOff.Checked) packetType = 98;
            if (rbText.Checked)
            {
                packetType = 116;
                length = (byte)txtLCD.Text.Length;
            }
            byte[] strBytes = Encoding.ASCII.GetBytes(txtLCD.Text);
            packet.Add(packetType);
            if (cbxCRC.Checked) length++;
            packet.Add(length);
            if (rbText.Checked) {
                packet.AddRange(strBytes);
            }
            if (cbxCRC.Checked)
            {
                byte crc = CRC8.ComputeChecksum(packet.ToArray());
                packet.Add(crc);
            }
            return packet.ToArray();
        }

        void showPacket()
        {
            lblPreview.Text = "";
            byte[] packet = createPacket();
            foreach (byte b in packet)
            {
                lblPreview.Text += b.ToString() + " ";
            }
        }

        private void txtLCD_TextChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbOn_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbOff_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbText_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void cmdSendRandom_Click(object sender, EventArgs e)
        {
            try
            {
                string[] strBytes = txtRandomBytes.Text.Split(' ');
                foreach (string str in strBytes)
                {
                    if (str.Length > 0)
                    {
                        byte[] b = new byte[1];
                        b[0] = Byte.Parse(str);
                        ComPort.Write(b, 0, 1);
                    }
                }
            } catch
            {
                MessageBox.Show("Nieprawidłowy ciąg bajtów", "Błąd", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void cmdOpen_Click(object sender, EventArgs e)
        {
            try
            {
                ComPort = new SerialPort(cmbPorts.SelectedItem.ToString());
                ComPort.BaudRate = 9600;
                ComPort.Parity = Parity.None;
                ComPort.StopBits = StopBits.One;
                ComPort.DataBits = 8;
                ComPort.Handshake = Handshake.None;
                ComPort.Open();
                ComPort.DataReceived += OnSerialDataReceived;
                cmdSendPacket.Enabled = true;
                cmdSendRandom.Enabled = true;
                cmdOpen.Enabled = false;
                cmdClose.Enabled = true;
            } catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Błąd otwierania portu", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void OnSerialDataReceived(object sender, SerialDataReceivedEventArgs args)
        {
            byte[] data = new byte[ComPort.BytesToRead];
            ComPort.Read(data, 0, data.Length);
            foreach(byte b in data) {
                txtReceived.Invoke((MethodInvoker)(() => txtReceived.Text += b.ToString() + " "));
            }
        }

        private void cmdClose_Click(object sender, EventArgs e)
        {
            cmdSendPacket.Enabled = false;
            cmdSendRandom.Enabled = false;
            cmdOpen.Enabled = true;
            cmdClose.Enabled = false;
            if (ComPort != null) ComPort.Close();
        }

        private void cmdClear_Click(object sender, EventArgs e)
        {
            txtReceived.Text = "";
        }

        private void cbxCRC_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }
    }
}
